# Generated by Django 3.1.5 on 2021-01-15 23:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_postalcodes_mexico', '0002_auto_20210115_2156'),
    ]

    operations = [
        migrations.AlterField(
            model_name='postalcode',
            name='d_zona',
            field=models.CharField(choices=[('Urbano', 'Urbano'), ('Rural', 'Rural'), ('Semiurbano', 'Semiurbano')], max_length=10, verbose_name='Tipo de Zona'),
        ),
    ]



# -*- coding: utf-8 -*-
from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _



class PostalCodeForm(forms.Form):
    postal_code = forms.CharField(label=_('Postal Code'))

    def clean_postal_code(self):
        postal_code = self.cleaned_data['postal_code']
        if not postal_code.isdigit():
            raise ValidationError(
                _("Please enter a valid postal code, only numbers are allowed"), code='invalid', params={'value': postal_code})
        if len(postal_code) != 5:
            raise ValidationError(
                _("A valid postal code must be 5 characters long"), code='invalid', params={'value': postal_code})
        return postal_code

class PostalCodeSearchForm(forms.Form):
    postal_code = forms.CharField(label=_('Postal Code'))

    def clean_postal_code(self):
        postal_code = self.cleaned_data['postal_code']
        if not postal_code.isdigit():
            raise ValidationError(
                _("Please enter a valid postal code, only numbers are allowed"), code='invalid', params={'value': postal_code})
        return postal_code


# -*- coding: utf-8 -*-
from django.http import JsonResponse
from django.utils.translation import ugettext_lazy as _

from .forms import PostalCodeForm, PostalCodeSearchForm
from .models import (
	PostalCode,
)
from .utils import generateCityAreasByPostalCode, searchPostalCode

def getPostalCodeData(request, postal_code):
    postalCodeForm = PostalCodeForm({"postal_code": postal_code})
    if postalCodeForm.is_valid():
        postal_code_data = generateCityAreasByPostalCode(postal_code)
        status=200 if postal_code_data else 404       
        return JsonResponse(postal_code_data, status=status)
    else:
        return JsonResponse(postalCodeForm.errors, status=400)

def searchSimilarPostalCode(request, postal_code):
    postalCodeForm = PostalCodeSearchForm({"postal_code": postal_code})
    if postalCodeForm.is_valid():
        postal_codes_list = searchPostalCode(postal_code)
        status=200 if postal_codes_list else 404       
        return JsonResponse(postal_codes_list, status=status)
    else:
        return JsonResponse(postalCodeForm.errors, status=400)    


# -*- coding: utf-8 -*-
import io
import sys
import zipfile
import xml.etree.ElementTree as ET

import requests

from ...models import PostalCode

node_prefix = '{NewDataSet}'
xml_postal_codes_url = 'https://www.correosdemexico.gob.mx/SSLServicios/ConsultaCP/CodigoPostal_Exportar.aspx'
postal_codes_node_tag = '{NewDataSet}table'
database_columns = ['d_codigo', 'd_asenta', 'D_mnpio', 'd_ciudad', 'd_CP', 'c_estado',
                   'c_oficina', 'c_tipo_asenta', 'c_mnpio', 'id_asenta_cpcons', 'd_zona', 'c_cve_ciudad']


def get_value_from_node(node, value):
    try:
        return node.find(value).text
    except:
        return ""


def generate_list_of_postalcode_objects(root, node_tag=postal_codes_node_tag):
    PostalCodes = []
    for node in root.findall(node_tag):
        data = {column: get_value_from_node(
            node, node_prefix + column) for column in database_columns}
        PostalCodes.append(data)
    return PostalCodes


def fetch_zipped_xml_file(url=xml_postal_codes_url):
    headers = {
        'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:84.0) Gecko/20100101 Firefox/84.0',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language': 'es-MX,es;q=0.8,en-US;q=0.5,en;q=0.3',
        'Content-Type': 'application/x-www-form-urlencoded',
        'Origin': 'https://www.correosdemexico.gob.mx',
        'Connection': 'keep-alive',
        'Referer': 'https://www.correosdemexico.gob.mx/SSLServicios/ConsultaCP/CodigoPostal_Exportar.aspx',
        'Upgrade-Insecure-Requests': '1',
        'Pragma': 'no-cache',
        'Cache-Control': 'no-cache',
    }
    # This data was obtained from the browser after a manual download.
    # Please note that it can change in the future
    data = {
        '__EVENTTARGET': '',
        '__EVENTARGUMENT': '',
        '__LASTFOCUS': '',
        '__VIEWSTATE': '/wEPDwUINzcwOTQyOTgPZBYCAgEPZBYCAgEPZBYGAgMPDxYCHgRUZXh0BTjDmmx0aW1hIEFjdHVhbGl6YWNpw7NuIGRlIEluZm9ybWFjacOzbjogRW5lcm8gMTYgZGUgMjAyMWRkAgcPEA8WBh4NRGF0YVRleHRGaWVsZAUDRWRvHg5EYXRhVmFsdWVGaWVsZAUFSWRFZG8eC18hRGF0YUJvdW5kZ2QQFSEjLS0tLS0tLS0tLSBUICBvICBkICBvICBzIC0tLS0tLS0tLS0OQWd1YXNjYWxpZW50ZXMPQmFqYSBDYWxpZm9ybmlhE0JhamEgQ2FsaWZvcm5pYSBTdXIIQ2FtcGVjaGUUQ29haHVpbGEgZGUgWmFyYWdvemEGQ29saW1hB0NoaWFwYXMJQ2hpaHVhaHVhEUNpdWRhZCBkZSBNw6l4aWNvB0R1cmFuZ28KR3VhbmFqdWF0bwhHdWVycmVybwdIaWRhbGdvB0phbGlzY28HTcOpeGljbxRNaWNob2Fjw6FuIGRlIE9jYW1wbwdNb3JlbG9zB05heWFyaXQLTnVldm8gTGXDs24GT2F4YWNhBlB1ZWJsYQpRdWVyw6l0YXJvDFF1aW50YW5hIFJvbxBTYW4gTHVpcyBQb3Rvc8OtB1NpbmFsb2EGU29ub3JhB1RhYmFzY28KVGFtYXVsaXBhcwhUbGF4Y2FsYR9WZXJhY3J1eiBkZSBJZ25hY2lvIGRlIGxhIExsYXZlCFl1Y2F0w6FuCVphY2F0ZWNhcxUhAjAwAjAxAjAyAjAzAjA0AjA1AjA2AjA3AjA4AjA5AjEwAjExAjEyAjEzAjE0AjE1AjE2AjE3AjE4AjE5AjIwAjIxAjIyAjIzAjI0AjI1AjI2AjI3AjI4AjI5AjMwAjMxAjMyFCsDIWdnZ2dnZ2dnZ2dnZ2dnZ2dnZ2dnZ2dnZ2dnZ2dnZ2dnZ2RkAh0PPCsACwBkGAEFHl9fQ29udHJvbHNSZXF1aXJlUG9zdEJhY2tLZXlfXxYBBQtidG5EZXNjYXJnYSesavHFx4/PcUhOXn0/O0E5DJnP',
        '__VIEWSTATEGENERATOR': 'BE1A6D2E',
        '__EVENTVALIDATION': '/wEWKAKHgaLWCwLG/OLvBgLWk4iCCgLWk4SCCgLWk4CCCgLWk7yCCgLWk7iCCgLWk7SCCgLWk7CCCgLWk6yCCgLWk+iBCgLWk+SBCgLJk4iCCgLJk4SCCgLJk4CCCgLJk7yCCgLJk7iCCgLJk7SCCgLJk7CCCgLJk6yCCgLJk+iBCgLJk+SBCgLIk4iCCgLIk4SCCgLIk4CCCgLIk7yCCgLIk7iCCgLIk7SCCgLIk7CCCgLIk6yCCgLIk+iBCgLIk+SBCgLLk4iCCgLLk4SCCgLLk4CCCgLL+uTWBALa4Za4AgK+qOyRAQLI56b6CwL1/KjtBRvavCjYcdNppWpaENjjAwloL6cu',
        'cboEdo': '00',
        'rblTipo': 'xml',
        'btnDescarga.x': '48',
        'btnDescarga.y': '2'
    }
    try:
        sys.stdout.write('Trying to connect to Mexican Postal Service (Correos de Mexico)\n')
        response = requests.post(url, headers=headers, data=data)
        sys.stdout.write('Response received\n')
    except Exception as e:
        sys.stderr.write(
            'There was a problem trying to get a response from Mexican Postal Service (Correos de Mexico): \n' + str(e))
    return response.content


def get_xml_postal_codes_data():
    with zipfile.ZipFile(io.BytesIO(fetch_zipped_xml_file())) as zipped_xml_file:
        with zipped_xml_file.open('CPdescarga.xml') as buffered_xml_file:
            try:
                root = ET.parse(buffered_xml_file).getroot()
            except Exception as e:
                sys.stderr.write(
                    'There was a problem parsing the xml data: \n' + str(e))
                exit(1)
    return root

def parse_xml_postal_codes_file(xml_file_name):
    try:
        xml_tree = ET.parse(xml_file_name).getroot()
    except Exception as e:
        sys.stderr.write(
            'There was a problem parsing the xml file, please make sure you downloaded it from Mexican Postal Service \'s official page.\n' + str(e))
        exit(1)
    return xml_tree



# -*- coding: utf-8 -*-
from django.urls import path
from django.views.generic import TemplateView

from . import views


app_name = 'django_postalcodes_mexico'
urlpatterns = [
    # There are postal codes with leading zeros, hence slug is prefered over int
    path('postal-code/<slug:postal_code>/', views.getPostalCodeData, name='get-postal-code-data'),
    path('postal-code-search/<slug:postal_code>/', views.searchSimilarPostalCode, name='search-postal-code-data')
]



XXXXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXXXXX
XXXXXXX
XXXXXXXXXXX
XXXXXXXXXX
XXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


# -*- coding: utf-8 -*-
import io
import zipfile
from django.core.management.base import BaseCommand, CommandError
import xml.etree.ElementTree as ET

import requests

from ...models import PostalCode
from ._utils import (
    get_value_from_node,
    generate_list_of_postalcode_objects,
    get_xml_postal_codes_data,
    parse_xml_postal_codes_file)


class Command(BaseCommand):
    help = 'Creates a postal code database using the xml data from the Mexican Postal Service (Correos de Mexico)'


    def add_arguments(self, parser):

        # Named (optional) arguments
        parser.add_argument(
            '--file',
            nargs='?',
            const='CPdescarga.xml',
            type=str,
            help='Specify the xml file that contains the postal codes',
        )

    def get_function_for_processing_xml_postal_codes(self, xml_file_name):
        if xml_file_name:
            return parse_xml_postal_codes_file(xml_file_name)
        return get_xml_postal_codes_data()

    def handle(self, *args, **options):
        try:
            xml_tree = self.get_function_for_processing_xml_postal_codes(options['file'])
        except Exception as e:
            self.stdout.write(self.style.ERROR(
                'There was an error obtaining the xml zipped file from Correos de Mexico:' + str(e)))
            exit(1)
        PostalCodes = []
        self.stdout.write(self.style.WARNING(
            'This process can take a few minutes, please be patient'))
        postalCodes = generate_list_of_postalcode_objects(xml_tree)
        self.stdout.write('Creating database...')
        PostalCode.objects.bulk_create(
            [PostalCode(**data) for data in postalCodes])
        self.stdout.write(self.style.SUCCESS(
            'The postal code database has been successfully populated'))
